@@ -19,6 +19,12 @@ from django.shortcuts import (
 )
 from django.template import RequestContext
 from django.template import defaultfilters as filters
+from django.utils.html import escape
+from django.utils.safestring import mark_safe
+
+from django_tables2 import Attrs, Column
+
+from lava.utils.data_tables.tables import DataTablesTable
 
 from lava_server.views import index as lava_index
 from lava_server.bread_crumbs import (
@@ -36,10 +42,6 @@ from lava_scheduler_app.models import (
     DeviceStateTransition,
     TestJob,
     )
-from lava_scheduler_app.tables import (
-    AjaxColumn,
-    AjaxTable,
-    )
 
 
 
@@ -51,7 +53,7 @@ def post_only(func):
     return decorated
 
 
-class DateColumn(AjaxColumn):
+class DateColumn(Column):
 
     def __init__(self, **kw):
         self._format = kw.get('date_format', settings.DATETIME_FORMAT)
@@ -61,42 +63,62 @@ class DateColumn(AjaxColumn):
         return filters.date(value, self._format)
 
 
-class IDLinkColumn(AjaxColumn):
+def pklink(record):
+    return mark_safe(
+        '<a href="%s">%s</a>' % (
+            record.get_absolute_url(),
+            escape(record.pk)))
+
+class IDLinkColumn(Column):
 
     def __init__(self, verbose_name="ID", **kw):
         kw['verbose_name'] = verbose_name
         super(IDLinkColumn, self).__init__(**kw)
 
     def render(self, record):
-        return '<a href="%s">%s</a>' % (record.get_absolute_url(), record.pk)
+        return pklink(record)
+
+
+class RestrictedIDLinkColumn(IDLinkColumn):
+
+    def render(self, record, table):
+        if record.is_accessible_by(table.context.get('request').user):
+            return pklink(record)
+        else:
+            return record.pk
 
 
 def all_jobs_with_device_sort():
     return TestJob.objects.select_related(
         "actual_device", "requested_device", "requested_device_type",
-        "submitter").extra(
+        "submitter", "user", "group").extra(
         select={
             'device_sort': 'coalesce(actual_device_id, requested_device_id, requested_device_type_id)'
             }).all()
 
 
-class JobTable(AjaxTable):
+class JobTable(DataTablesTable):
 
     def render_device(self, record):
         if record.actual_device:
-            return '<a href="%s">%s</a>' % (
-                record.actual_device.get_absolute_url(), record.actual_device.pk)
+            return pklink(record.actual_device)
         elif record.requested_device:
-            return '<a href="%s">%s</a>' % (
-                record.requested_device.get_absolute_url(), record.requested_device.pk)
+            return pklink(record.requested_device)
         else:
-            return '<i>' + record.requested_device_type.pk + '</i>'
+            return mark_safe(
+                '<i>' + escape(record.requested_device_type.pk) + '</i>')
 
-    id = IDLinkColumn()
-    status = AjaxColumn()
-    device = AjaxColumn(sort_expr='device_sort')
-    description = AjaxColumn(width="30%")
-    submitter = AjaxColumn(accessor='submitter.username')
+    def render_description(self, value):
+        if value:
+            return value
+        else:
+            return ''
+
+    id = RestrictedIDLinkColumn()
+    status = Column()
+    device = Column(accessor='device_sort')
+    description = Column(attrs=Attrs(width="30%"))
+    submitter = Column()
     submit_time = DateColumn()
     end_time = DateColumn()
 
@@ -107,29 +129,33 @@ class JobTable(AjaxTable):
 
 
 class IndexJobTable(JobTable):
+    def get_queryset(self):
+        return all_jobs_with_device_sort().filter(
+            status__in=[TestJob.SUBMITTED, TestJob.RUNNING])
+
     class Meta:
         exclude = ('end_time',)
 
 
 def index_active_jobs_json(request):
-    return IndexJobTable.json(
-        request, all_jobs_with_device_sort().filter(
-            status__in=[TestJob.SUBMITTED, TestJob.RUNNING]))
+    return IndexJobTable.json(request)
 
 
-class DeviceTable(AjaxTable):
+class DeviceTable(DataTablesTable):
+
+    def get_queryset(self):
+        return Device.objects.select_related("device_type")
 
     hostname = IDLinkColumn("hostname")
-    device_type = AjaxColumn(accessor='device_type.pk')
-    status = AjaxColumn()
-    health_status = AjaxColumn()
+    device_type = Column()
+    status = Column()
+    health_status = Column()
 
     searchable_columns=['hostname']
 
 
 def index_devices_json(request):
-    return DeviceTable.json(
-        request, Device.objects.select_related("device_type"))
+    return DeviceTable.json(request)
 
 
 @BreadCrumb("Scheduler", parent=lava_index)
@@ -145,22 +171,34 @@ def index(request):
         RequestContext(request))
 
 
-class DeviceHealthTable(AjaxTable):
+def get_restricted_job(user, pk):
+    return get_object_or_404(
+        TestJob.objects.accessible_by_principal(user), pk=pk)
+
+
+class DeviceHealthTable(DataTablesTable):
+
+    def get_queryset(self):
+        return Device.objects.select_related(
+            "hostname", "last_health_report_job")
 
     def render_hostname(self, record):
-        return '<a href="%s">%s</a>' % (record.get_device_health_url(), record.pk)
+        return mark_safe('<a href="%s">%s</a>' % (
+            record.get_device_health_url(), escape(record.pk)))
 
-    def render_last_report_job(self, record):
+    def render_last_health_report_job(self, record):
         report = record.last_health_report_job
         if report is None:
             return ''
         else:
-            return '<a href="%s">%s</a>' % (report.get_absolute_url(), report.pk)
+            return pklink(report)
 
-    hostname = AjaxColumn("hostname")
-    health_status = AjaxColumn()
-    last_report_time = DateColumn(accessor="last_health_report_job.end_time")
-    last_report_job = AjaxColumn()
+    hostname = Column("hostname")
+    health_status = Column()
+    last_report_time = DateColumn(
+        verbose_name="last report time",
+        accessor="last_health_report_job.end_time")
+    last_health_report_job = Column("last report job")
 
     searchable_columns=['hostname']
     datatable_opts = {
@@ -169,9 +207,7 @@ class DeviceHealthTable(AjaxTable):
 
 
 def lab_health_json(request):
-    return DeviceHealthTable.json(
-        request, Device.objects.select_related(
-            "hostname", "last_health_report_job"))
+    return DeviceHealthTable.json(request)
 
 
 @BreadCrumb("All Device Health", parent=index)
@@ -187,19 +223,21 @@ def lab_health(request):
 
 
 class HealthJobTable(JobTable):
+
+    def get_queryset(self, device):
+        return TestJob.objects.select_related(
+            "submitter",
+            ).filter(
+            actual_device=device,
+            health_check=True)
+
     class Meta:
         exclude = ('description', 'device')
 
 
-
 def health_jobs_json(request, pk):
     device = get_object_or_404(Device, pk=pk)
-    return HealthJobTable.json(
-        request, TestJob.objects.select_related(
-            "submitter",
-        ).filter(
-            actual_device=device,
-            health_check=True))
+    return HealthJobTable.json(params=(device,))
 
 
 @BreadCrumb("All Health Jobs on Device {pk}", parent=index, needs=['pk'])
@@ -210,8 +248,12 @@ def health_job_list(request, pk):
         "lava_scheduler_app/health_jobs.html",
         {
             'device': device,
+            'transition_table': DeviceTransitionTable(
+                'transitions', reverse(transition_json, kwargs=dict(pk=device.pk)),
+                params=(device,)),
             'health_job_table': HealthJobTable(
-                'health_jobs', reverse(health_jobs_json, kwargs=dict(pk=pk))),
+                'health_jobs', reverse(health_jobs_json, kwargs=dict(pk=pk)),
+                params=(device,)),
             'show_maintenance': device.can_admin(request.user) and \
                 device.status in [Device.IDLE, Device.RUNNING],
             'show_online': device.can_admin(request.user) and \
@@ -223,6 +265,9 @@ def health_job_list(request, pk):
 
 class AllJobsTable(JobTable):
 
+    def get_queryset(self):
+        return all_jobs_with_device_sort()
+
     datatable_opts = JobTable.datatable_opts.copy()
 
     datatable_opts.update({
@@ -231,8 +276,7 @@ class AllJobsTable(JobTable):
 
 
 def alljobs_json(request):
-    return AllJobsTable.json(
-        request, all_jobs_with_device_sort())
+    return AllJobsTable.json(request)
 
 
 @BreadCrumb("All Jobs", parent=index)
@@ -248,7 +292,7 @@ def job_list(request):
 
 @BreadCrumb("Job #{pk}", parent=index, needs=['pk'])
 def job_detail(request, pk):
-    job = get_object_or_404(TestJob, pk=pk)
+    job = get_restricted_job(request.user, pk)
 
     data = {
         'job': job,
@@ -285,7 +329,7 @@ def job_detail(request, pk):
 
 
 def job_definition(request, pk):
-    job = get_object_or_404(TestJob, pk=pk)
+    job = get_restricted_job(request.user, pk)
     return render_to_response(
         "lava_scheduler_app/job_definition.html",
         {
@@ -296,7 +340,7 @@ def job_definition(request, pk):
 
 
 def job_definition_plain(request, pk):
-    job = get_object_or_404(TestJob, pk=pk)
+    job = get_restricted_job(request.user, pk)
     response = HttpResponse(job.definition, mimetype='text/plain')
     response['Content-Disposition'] = "attachment; filename=job_%d.json"%job.id
     return response
@@ -304,7 +348,7 @@ def job_definition_plain(request, pk):
 
 @BreadCrumb("Complete log", parent=job_detail, needs=['pk'])
 def job_log_file(request, pk):
-    job = get_object_or_404(TestJob, pk=pk)
+    job = get_restricted_job(request.user, pk)
     content = formatLogFile(job.log_file)
     return render_to_response(
         "lava_scheduler_app/job_log_file.html",
@@ -318,7 +362,7 @@ def job_log_file(request, pk):
 
 
 def job_log_file_plain(request, pk):
-    job = get_object_or_404(TestJob, pk=pk)
+    job = get_restricted_job(request.user, pk)
     response = HttpResponse(job.log_file, mimetype='text/plain')
     response['Content-Disposition'] = "attachment; filename=job_%d.log"%job.id
     return response
@@ -326,7 +370,7 @@ def job_log_file_plain(request, pk):
 
 def job_log_incremental(request, pk):
     start = int(request.GET.get('start', 0))
-    job = get_object_or_404(TestJob, pk=pk)
+    job = get_restricted_job(request.user, pk)
     log_file = job.log_file
     log_file.seek(start)
     new_content = log_file.read()
@@ -341,7 +385,7 @@ def job_log_incremental(request, pk):
 
 def job_full_log_incremental(request, pk):
     start = int(request.GET.get('start', 0))
-    job = get_object_or_404(TestJob, pk=pk)
+    job = get_restricted_job(request.user, pk)
     log_file = job.log_file
     log_file.seek(start)
     new_content = log_file.read()
@@ -368,7 +412,7 @@ def job_output(request, pk):
     except ValueError:
         return HttpResponseBadRequest("invalid start")
     count_present = 'count' in request.GET
-    job = get_object_or_404(TestJob, pk=pk)
+    job = get_restricted_job(request.user, pk)
     log_file = job.log_file
     log_file.seek(0, os.SEEK_END)
     size = int(request.GET.get('count', log_file.tell()))
@@ -397,7 +441,7 @@ def job_output(request, pk):
 
 @post_only
 def job_cancel(request, pk):
-    job = get_object_or_404(TestJob, pk=pk)
+    job = get_restricted_job(request.user, pk)
     if job.can_cancel(request.user):
         job.cancel()
         return redirect(job)
@@ -407,7 +451,7 @@ def job_cancel(request, pk):
 
 
 def job_json(request, pk):
-    job = get_object_or_404(TestJob, pk=pk)
+    job = get_restricted_job(request.user, pk)
     json_text = simplejson.dumps({
         'status': job.get_status_display(),
         'results_link': job.results_link,
@@ -420,16 +464,31 @@ def job_json(request, pk):
 
 
 class RecentJobsTable(JobTable):
+
+    def get_queryset(self, device):
+        return device.recent_jobs()
+
     class Meta:
         exclude = ('device',)
 
 
 def recent_jobs_json(request, pk):
     device = get_object_or_404(Device, pk=pk)
-    return RecentJobsTable.json(request, device.recent_jobs())
+    return RecentJobsTable.json(request, params=(device,))
 
 
-class DeviceTransitionTable(AjaxTable):
+class DeviceTransitionTable(DataTablesTable):
+
+    def get_queryset(self, device):
+        qs = device.transitions.select_related('created_by')
+        qs = qs.extra(select={'prev': """
+        select t.created_on
+          from lava_scheduler_app_devicestatetransition as t
+         where t.device_id=%s and t.created_on < lava_scheduler_app_devicestatetransition.created_on
+         order by t.created_on desc
+         limit 1 """},
+                      select_params=[device.pk])
+        return qs
 
     def render_created_on(self, record):
         t = record
@@ -440,7 +499,8 @@ class DeviceTransitionTable(AjaxTable):
 
     def render_transition(self, record):
         t = record
-        return '%s &rarr; %s' % (t.get_old_state_display(), t.get_new_state_display(),)
+        return mark_safe(
+            '%s &rarr; %s' % (t.get_old_state_display(), t.get_new_state_display(),))
 
     def render_message(self, value):
         if value is None:
@@ -448,10 +508,10 @@ class DeviceTransitionTable(AjaxTable):
         else:
             return value
 
-    created_on = AjaxColumn('when', width="40%")
-    transition = AjaxColumn('transition', sortable=False)
-    created_by = AjaxColumn('by', accessor='created_by.username')
-    message = AjaxColumn('reason')
+    created_on = Column('when', attrs=Attrs(width="40%"))
+    transition = Column('transition', sortable=False)
+    created_by = Column('by')
+    message = Column('reason')
 
     datatable_opts = {
         'aaSorting': [[0, 'desc']],
@@ -460,16 +520,7 @@ class DeviceTransitionTable(AjaxTable):
 
 def transition_json(request, pk):
     device = get_object_or_404(Device, pk=pk)
-    qs = device.transitions.select_related('created_by')
-    qs = qs.extra(select={'prev': """
-    select t.created_on
-      from lava_scheduler_app_devicestatetransition as t
-     where t.device_id=%s and t.created_on < lava_scheduler_app_devicestatetransition.created_on
-     order by t.created_on desc
-     limit 1 """},
-                  select_params=[device.pk])
-    return DeviceTransitionTable.json(request, qs)
-
+    return DeviceTransitionTable.json(request, params=(device,))
 
 
 @BreadCrumb("Device {pk}", parent=index, needs=['pk'])
@@ -488,9 +539,11 @@ def device_detail(request, pk):
             'device': device,
             'transition': transition,
             'transition_table': DeviceTransitionTable(
-                'transitions', reverse(transition_json, kwargs=dict(pk=device.pk))),
+                'transitions', reverse(transition_json, kwargs=dict(pk=device.pk)),
+                params=(device,)),
             'recent_job_table': RecentJobsTable(
-                'jobs', reverse(recent_jobs_json, kwargs=dict(pk=device.pk))),
+                'jobs', reverse(recent_jobs_json, kwargs=dict(pk=device.pk)),
+                params=(device,)),
             'show_maintenance': device.can_admin(request.user) and \
                 device.status in [Device.IDLE, Device.RUNNING],
             'show_online': device.can_admin(request.user) and \
